//Behaviour.cpp

//This is the implementation file for the behaviour.

#include "Behaviour.h"

//Entities are represented by BMessages which contain all the attributes of a given entity.
//The behaviour algorithm can inspect the environment before deciding what changes to make
//to the attributes.

//BList *prev
//This is a list of pointers to BMessages. Each represents the state of an entity as it
//appeared at the end of the previous generation. This is the "environment".

//BList *next
//This is a list of entities as they should be in the next generation.
//Initially this list is a copy of "prev". The simulator inspects each entity to see what
//behaviours it has. For each behaviour it calls the Behaviour() function of the appropriate
//add-on with the "index" of that entity as a parameter.

//int32 index
//The index of the current entity in the "prev" and "next" lists.
//i.e., current = next->ItemAt(index);

//BList *children
//One possible behaviour is the creation of additional entities. This is done by
//adding the entities (BMessages) to the "children" list.

#include <Message.h>
#include <List.h>

status_t Behaviour(const BList *prev, const BList *next, int32 index, BList *children)
{

#if __INTEL__
	const int32 BLUE = 0xFFFF0000;	//ABGR
	const int32 RED = 0xFF0000FF;
	const int32 GREEN = 0xFF00FF00;
#endif

#if __POWERPC__
	const int32 BLUE = 0x0000FFFF;	//RGBA
	const int32 RED = 0xFF0000FF;
	const int32 GREEN = 0x00FF00FF;
#endif

	const int32 WIDTH = 25;

	BMessage *entity = (BMessage *)next->ItemAt(index);

	//Get the details we are interested in
	float x = entity->FindFloat("x-pos");
	float y = entity->FindFloat("y-pos");
	float z = entity->FindFloat("z-pos");
	int32 color = entity->FindInt32("color");

	//Do whatever changes we like

	switch(color)
	{
		case RED:
		{
			if (x > WIDTH/3)
				x--;
		}
		break;
	
		case GREEN:
		{
			if (x <= WIDTH/3)
				x++;
			else if (x >= (2*WIDTH)/3)
				x--;
		}
		break;

		case BLUE:
		{
			if (x < (2*WIDTH)/3)
				x++;
		}
		break;
	}	

	if (y > 0)
		y--;
	else if (y < 0)
		y++;

	//Write the details back to the BMessage
	entity->ReplaceFloat("x-pos", x);
	entity->ReplaceFloat("z-pos", z);
	entity->ReplaceFloat("y-pos", y);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
