//Behaviour.cpp

//This is the implementation file for the behaviour.

#include "Behaviour.h"

//Entities are represented by BMessages which contain all the attributes of a given entity.
//The behaviour algorithm can inspect the environment before deciding what changes to make
//to the attributes.

//BList *prev
//This is a list of pointers to BMessages. Each represents the state of an entity as it
//appeared at the end of the previous generation. This is the "environment".

//BList *next
//This is a list of entities as they should be in the next generation.
//Initially this list is a copy of "prev". The simulator inspects each entity to see what
//behaviours it has. For each behaviour it calls the Behaviour() function of the appropriate
//add-on with the "index" of that entity as a parameter.

//int32 index
//The index of the current entity in the "prev" and "next" lists.
//i.e., current = next->ItemAt(index);

//BList *children
//One possible behaviour is the creation of additional entities. This is done by
//adding the entities (BMessages) to the "children" list.

status_t Behaviour(const BList *prev, const BList *next, int32 index, BList *children)
{
	BMessage *entity = (BMessage *)next->ItemAt(index);

	float y;
	int32 y_direction;
	status_t err;
	
	//Get the details we are interested in
	if ((err = entity->FindFloat("y-pos", &y)) < B_OK)
	{
		//if it doesn't have the attribute add it
		entity->AddFloat("y-pos", 0);
		y = 0;
	}
	
	if ((err = entity->FindInt32("y-direction", &y_direction)) < B_OK)
	{
		//if it doesn't have the attribute add it
		entity->AddInt32("y-direction", 0);
		y_direction = 1;
	}

	//Make it go from side-to-side
	if (y >= 25)
		y_direction = -1;
	else if (y <= 0)
		y_direction = 1;

	if (y_direction == 1)
		y += 0.25;
	else
		y -= 0.25;		
	
	//Write the details back to the BMessage
	entity->ReplaceFloat("y-pos", y);
	entity->ReplaceInt32("y-direction", y_direction);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
