//addonManager.cpp
//Copyright © 1998, Paul Yu 

#include <stdio.h>
#include <StorageKit.h>
#include "addonManager.h"

addonManager::addonManager()
{
	app_info ai;
	BDirectory dir;
	BDirectory sub_dir;
	BPath path;
	BEntry entry;
	image_id id;
	void *function;
	
	be_app->GetAppInfo(&ai);
	if ((entry.SetTo(&ai.ref) == B_OK) && (entry.GetParent(&dir) == B_OK))
	{
		//Behaviours
		fprintf(stderr, "\nBehaviour add-ons:\n");
		if (sub_dir.SetTo(&dir, "add-ons/behaviours") == B_OK)
		{
			while(sub_dir.GetNextEntry(&entry, true) == B_OK)
			{
				if ((entry.GetPath(&path) == B_OK) &&
					((id = load_add_on(path.Path())) > 0))
				{
					if ((get_image_symbol(id, "Behaviour",
							B_SYMBOL_TYPE_TEXT, &function) < B_OK) ||
						 (mBehaviours.AddItem(path.Leaf(), function) < B_OK))
					{
						unload_add_on(id);
					}
					else
						mImages.AddItem((void *)id);
				}
			}
		}

		//Visuals
		printf("\nVisual appearance add-ons:\n");
		if (sub_dir.SetTo(&dir, "add-ons/visuals") == B_OK)
		{
			while(sub_dir.GetNextEntry(&entry, true) == B_OK)
			{
				if ((entry.GetPath(&path) == B_OK) &&
					((id = load_add_on(path.Path())) > 0))
				{
					if ((get_image_symbol(id, "Draw",
							B_SYMBOL_TYPE_TEXT, &function) < B_OK) ||
						 (mVisuals.AddItem(path.Leaf(), function) < B_OK))
					{
						unload_add_on(id);
					}
					else
						mImages.AddItem((void *)id);
				}
			}
		}	

		//Initialisers
		printf("\nInitialiser add-ons:\n");
		if (sub_dir.SetTo(&dir, "add-ons/initialisers") == B_OK)
		{
			while(sub_dir.GetNextEntry(&entry, true) == B_OK)
			{
				if ((entry.GetPath(&path) == B_OK) &&
					((id = load_add_on(path.Path())) > 0))
				{
					if ((get_image_symbol(id, "Initialise",
							B_SYMBOL_TYPE_TEXT, &function) < B_OK) ||
						 (mInitialisers.AddItem(path.Leaf(), function) < B_OK))
					{
						unload_add_on(id);
					}
					else
						mImages.AddItem((void *)id);
				}
			}
		}	
	}
}

addonManager::~addonManager()
{
	printf("addonManager::Destructor()\n");

	//Don't really need to do this for an application
	int32 count = mImages.CountItems();
	for(int32 i = 0; i < count; i++)
	{
		unload_add_on((image_id)mImages.ItemAt(i));
	}
}

void *addonManager::GetBehaviour(const char *algorithm)
{
	return mBehaviours.FindItem(algorithm);
}

void *addonManager::GetVisual(const char *algorithm)
{
	return mVisuals.FindItem(algorithm);
}

void *addonManager::GetInitialiser(const char *algorithm)
{
	return mInitialisers.FindItem(algorithm);
}

const char *addonManager::GetBehaviour(int32 index)
{
	return mBehaviours.GetName(index);
}

const char *addonManager::GetVisual(int32 index)
{
	return mVisuals.GetName(index);
}

const char *addonManager::GetInitialiser(int32 index)
{
	return mInitialisers.GetName(index);
}
