//entityWindow.cpp
//Copyright © 1998, Paul Yu 

#include <InterfaceKit.h>
#include "entityWindow.h"
#include "listView.h"
#include "navView.h"
#include "controlView.h"
#include "message_protocols.h"
#include "addonManager.h"
#include "attributeWindow.h"

entityWindow::entityWindow() : 
	BWindow(BRect(100,100,200,800), "", B_TITLED_WINDOW_LOOK, B_MODAL_SUBSET_WINDOW_FEEL, B_NOT_RESIZABLE)
{
	/*	*/
}

entityWindow::~entityWindow()
{
	/*	*/
}

void entityWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case B_OK:
		{
			BMessage message(UPDATE);
			BMessenger messenger(mEditorWindow);
			
			mEntity.RemoveName("name");
			if ((mNameInput->Text() != NULL) && (*mNameInput->Text() != '\0'))
				mEntity.AddString("name", mNameInput->Text());
			
			message.AddMessage("entity", &mEntity);
			message.AddInt32("index", mIndex);
			messenger.SendMessage(&message);

			Quit();
		}
		break;
		
		case B_CANCEL:
		{
			Quit();
		}
		break;

		case ADD:
		{
			listView *list;
			if (msg->FindPointer("listview", (void **)&list) == B_OK)
			{
				if (list == mBehavioursList)
				{
					Editor("behaviours");
				}
				else if (list == mVisualsList)
				{
					Editor("visuals");
				}
				else if (list == mInitialisersList)
				{
					Editor("initialisers");
				}
			}
		}
		break;

		case UPDATE:
		{
			const char *type = msg->FindString("type");
			const char *attribute;
			listView *list;
			
			if (strcmp(type, "behaviours") == 0)
			{
				list = mBehavioursList;
			}
			else if (strcmp(type, "visuals") == 0)
			{
				list = mVisualsList;
			}
			else if (strcmp(type, "initialisers") == 0)
			{
				list = mInitialisersList;
			}
			else
				return;
				
			int32 i = 0;
			while(msg->FindString("text", i++, &attribute) == B_OK)
			{
				list->AddItem(attribute);			
				mEntity.AddString(type, attribute);
			}
		}
		break;
		
		case EDIT:
		{
			listView *list;
			if (msg->FindPointer("listview", (void **)&list) == B_OK)
			{
//				int32 index = list->GetSelection(NULL);
//				
//				if (index >= 0)
//				{
					//Unimplemented
					BAlert *alert = new BAlert("", "This attribute has no configurable options.", "OK");
					alert->Go();
//				}
			}
		}
		break;

		case DELETE:
		{
			listView *list;

			if (msg->FindPointer("listview",(void **) &list) == B_OK)
			{
				int32 index = list->GetSelection(NULL);
				
				if (index < 0)
					return;
					
				list->RemoveItem();

				if (list == mBehavioursList)
				{
					mEntity.RemoveData("behaviours", index);
				}
				else if (list == mVisualsList)
				{
					mEntity.RemoveData("visuals", index);
				}
				else if (list == mInitialisersList)
				{
					mEntity.RemoveData("initialisers", index);
				}
			}
		}
		break;

		case SET_CONTEXT:
		{
			if ((msg->FindPointer("add-on_manager", (void **)&mAddons) < B_OK) ||
				(msg->FindPointer("editor_window", (void **)&mEditorWindow) < B_OK) ||
				(msg->FindInt32("index", &mIndex) < B_OK) ||
				(msg->FindMessage("entity", &mEntity) < B_OK))
					Quit();
			AddToSubset(mEditorWindow);
			SetTitle(mEntity.FindString("name"));
			
			if (Title() == NULL)
				SetTitle("Unnamed");
			
			Show();
		}
		break;	
	
		default:
			BWindow::MessageReceived(msg);
	}
}

void entityWindow::Show()
{
	const char *attribute;

	//Name Input
	BView *backview = new BView(BRect(0,0,370,34), "", B_FOLLOW_NONE, B_WILL_DRAW);
	backview->SetViewColor(0xe0, 0xe0, 0xe0, 0xff);
	mNameInput = new BTextControl(BRect(16,10,200,34), "Name",
		"Name", mEntity.FindString("name"), NULL);
	mNameInput->SetDivider(50);
	mNameInput->SetViewColor(0xe0, 0xe0, 0xe0, 0);
	backview->AddChild(mNameInput);
	AddChild(backview);

	//Behaviours
	mBehavioursList = new listView();
	mBehavioursList->SetListTitle("Behaviours");

	int32 i = 0;
	while(mEntity.FindString("behaviours", i++, &attribute) == B_OK)
	{
		mBehavioursList->AddItem(attribute);
	}
	AddChild(mBehavioursList);

	//Bug
	//AddChild(new controlView());
	controlView *bug = new controlView(mBehavioursList);
	AddChild(bug);
	bug->MoveTo(261, 35);
	//

	//Visuals
	mVisualsList = new listView();
	mVisualsList->SetListTitle("Visual Appearance");

	i = 0;
	while(mEntity.FindString("visuals", i++, &attribute) == B_OK)
	{
		mVisualsList->AddItem(attribute);
	}
	AddChild(mVisualsList);
	AddChild(new controlView(mVisualsList));

	//Initialisers
	mInitialisersList = new listView();
	mInitialisersList->SetListTitle("Initialisers");

	i = 0;
	while(mEntity.FindString("initialisers", i++, &attribute) == B_OK)
	{
		mInitialisersList->AddItem(attribute);
	}
	AddChild(mInitialisersList);
	AddChild(new controlView(mInitialisersList));

	//Navigation
	AddChild(new navView());
	
	BWindow::Show();
}

void entityWindow::Editor(const char *type)
{
	BMessage msg(SET_CONTEXT);

	if ((msg.AddPointer("add-on_manager", mAddons) < B_OK) ||
		(msg.AddPointer("entity_window", this) < B_OK) ||
		(msg.AddPointer("editor_window", mEditorWindow) < B_OK) ||
		(msg.AddString("type", type) < B_OK))
		return;
		
	attributeWindow *window = new attributeWindow();
	window->Run();
	window->PostMessage(&msg);
}
