//listView.cpp
//Copyright © 1998, Paul Yu 

#include <InterfaceKit.h>
#include "listView.h"

static bool Delete(BListItem *object);

listView::listView() : BView(BRect(1,0,261,179), "listView",
	B_FOLLOW_NONE, B_WILL_DRAW)
{
	SetViewColor(0xe0, 0xe0, 0xe0, 0);
	BBox *bbox = new BBox(BRect(10,10,250,180), "BBox", B_FOLLOW_NONE);
	bbox->SetLabel("Label");
	mListView = new BListView(BRect(5,15,220,165),
		"BListView", B_SINGLE_SELECTION_LIST, B_FOLLOW_NONE);
	bbox->AddChild(mListView);

	BScrollBar *scrollbar = new BScrollBar(
		BRect(221, 15, 221 + B_V_SCROLL_BAR_WIDTH, 165), "BScrollBar",
			mListView, 0,0, B_VERTICAL);
	bbox->AddChild(scrollbar);
	AddChild(bbox);
}

listView::~listView()
{
	mListView->DoForEach(&Delete);
}

void listView::AttachedToWindow()
{
	int32 y = 0;
	BWindow *window = Window();
	BRect rect = window->Bounds();

	while(window->FindView(BPoint(0, y)) != NULL)
		y += 5;

	MoveTo(0, y);

	if (rect.Width() < Bounds().Width())
		window->ResizeTo(Bounds().Width(), rect.Height());
}

void listView::MakeEmpty()
{
	mListView->DoForEach(&Delete);
	mListView->MakeEmpty();
}

void listView::AddItem(const char *text)
{
	BStringItem *item = new BStringItem(text);
	mListView->AddItem(item);
}

void listView::AddItem(const char *text, int32 index)
{
	BStringItem *item = new BStringItem(text);
	mListView->AddItem(item, index);
}

void listView::RemoveItem()
{
	int32 index = mListView->CurrentSelection();

	mListView->RemoveItem(index);
	mListView->Select(index);
}

void listView::ReplaceItem(const char *text, int32 index)
{
	BStringItem *item = (BStringItem *)mListView->ItemAt(index);
	item->SetText(text);
	mListView->InvalidateItem(index);
}

void listView::SetListTitle(const char *text)
{
	BBox *box = dynamic_cast<BBox *>(FindView("BBox"));

	if (box)
	{
		box->SetLabel(text);
	}
}

void listView::SetListType(list_view_type type)
{
	mListView->SetListType(type);
}

int32 listView::GetSelection(BMessage *msg)
{
	int32 index;
	int32 i = 0;
	BStringItem *item;

	if (msg)
	{
		while((index = mListView->CurrentSelection(i++)) >= 0)
		{
			item = (BStringItem *)mListView->ItemAt(index);
			msg->AddString("text", item->Text());
			msg->AddInt32("index", index);
		}
	}

	return mListView->CurrentSelection(0);
}

static bool Delete(BListItem *object)
{
	delete object;
	return false;
}
