//Draw.cpp

//This is the implementation file for the drawing algorithm.

#include "Draw.h"
#include <opengl/GLView.h>

//Entities are represented by BMessages which contain all the attributes of a given entity.
//Initialisers are used to give entities some default attributes.

status_t Draw(BMessage *entity)
{
	//Get the attributes
	float x = entity->FindFloat("x-pos");
	float y = entity->FindFloat("y-pos");
	float z = entity->FindFloat("z-pos");

	rgb_color color;
	if (entity->FindInt32("color", (int32 *)&color) < B_OK)
	{
		color.red = 255;
		color.blue = 0;
		color.green = 0;
		color.alpha = 255;	
	}

	//OpenGL stuff... draws a cube apparently.
	float fx,fy,fz;
	fx = x + x*0.2;  // some space between cubes
	fy = y + y*0.2;
	fz = z + z*0.2;

	GLfloat vertices[] = {fx, fy, fz,
							fx, fy+1, fz,
							fx+1, fy+1, fz,
							fx+1, fy, fz,
							fx, fy, fz+1, 
							fx, fy+1, fz+1,
							fx+1, fy+1, fz+1,
							fx+1, fy, fz+1};

	
	// each group of four vertices is a face of the cube
	static GLubyte allIndices[] = { 0, 1, 2, 3,
									3, 2, 6, 7,
									7, 6, 5, 4,
									4, 5, 1, 0,
									1, 5, 6, 2,
									3, 7, 4, 0 };

	glColor4f(color.red/255, color.green/255, color.blue/255, color.alpha/255);

	// draw all sides of the cube
	glVertexPointer(3, GL_FLOAT, 0, vertices);
	glDrawElements(GL_QUADS, 24, GL_UNSIGNED_BYTE, allIndices);

	// add a black outline to the cube
	glColor3f(0,0,0);
	glDrawElements(GL_LINE_LOOP, 24, GL_UNSIGNED_BYTE, allIndices);

	return B_OK;
}

//don't forget to go Window->Settings->x86 Project and change the default file name
